############################################
# Purpose:
# This script can be used to remove all the contents of a Fortran code that can make it harder to parse.
# It will merge split lines, remove comments and empty lines.
#
# Inputs:
#
# "-s", "--input-source" path to a source-code folder which code will be cleaned.
#
#
############################################
#
from AutoRPE.UtilsRPE.CodeTools import expand_sources
from AutoRPE.UtilsRPE.Functions import remove_comments


def lines_from_filename(filename):
    """
    Opens file and returns its lines stripped.
    """
    with open(filename, "r", encoding="latin-1") as f:
        # Read, strip and keep only lines with content
        _lines = [_l.strip() for _l in f if _l.strip()]
    return _lines


def save_lines_into_file(_lines, filename):
    """
    Saves a list of lines into a file with a given filename.
    """
    to_write = "\n".join(_lines).strip()
    with open(filename, "w") as f:
        f.write(to_write)


def merge_lines(_lines):
    """
    Looks for split lines and merges them.
    """
    for _index, _line in enumerate(_lines):

        line = _lines[_index]
        if not line:
            continue
        if line[0] == "#":
            continue
        if line.count("&"):
            i = 1
            while line and line.strip()[-1] == "&":
                line1 = line
                line2 = _lines[_index + i]
                line = line1.replace("&", "").rstrip() + line2
                i += 1
            line = line.replace("&", "")
            _lines[_index] = line
            for j in range(1, i):
                _lines[_index + j] = ""
    return _lines


def remove_empty_lines(_lines):
    """
    Remove empty strings from list of strings
    """
    return [_l for _l in _lines if _l.strip()]


if __name__ == "__main__":
    # Use OptionParser to parse command line arguments.
    from optparse import OptionParser
    parser = OptionParser()

    parser.add_option("-s", "--input-source", dest="input_source",
                      default=None, metavar="FILE")

    (options, args) = parser.parse_args()

    path = options.input_source

    if path is None:
        parser.print_help()
        exit(0)

    # Find all files inside the provided path
    files = expand_sources(path)

    # Loop over files
    for index, fn in enumerate(files):
        # Read file
        lines = lines_from_filename(fn)
        # Remove comments ignoring preprocessor statements
        for line_index, line in enumerate(lines):
            if line[0] == "#":
                continue
            lines[line_index] = remove_comments(line)
        # Remove empty lines
        lines = remove_empty_lines(lines)

        # Merge lines
        lines = merge_lines(lines)
        # Remove empty lines again
        lines = remove_empty_lines(lines)
        # Overwrite file with clean source
        save_lines_into_file(lines, fn)

